package com.drinkertea.test2smack;

import androidx.appcompat.app.AppCompatActivity;

import android.graphics.Color;
import android.graphics.Typeface;
import android.os.Bundle;
import android.os.SystemClock;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

import org.jivesoftware.smack.AbstractXMPPConnection;
import org.jivesoftware.smack.chat2.Chat;
import org.jivesoftware.smack.chat2.ChatManager;
import org.jivesoftware.smack.chat2.IncomingChatMessageListener;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smackx.mam.MamManager;
import org.jxmpp.jid.EntityBareJid;
import org.jxmpp.jid.impl.JidCreate;
import org.jxmpp.stringprep.XmppStringprepException;

import java.util.List;

public class ChatActivity extends AppCompatActivity {

    TextView tView;
    AbstractXMPPConnection connect = MainActivity.connection;
    ChatManager chatManager = MainActivity.chatManager;
    LinearLayout chatLayout;
    EntityBareJid jid;

    {
        try {
            jid = ChatsList.jid;
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    Chat chat = chatManager.chatWith(jid);

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_chat);
        tView = (TextView) findViewById(R.id.testView);
        chatLayout = (LinearLayout) findViewById(R.id.chatLayout);
        createMessageForm();

    }
    public void sendMessage(View view) throws Exception{
        String text = tView.getText().toString();

        chat.send(text);
        addMessage(connect.getUser().asBareJid().toString(), text, false);
        tView.setText("");
    }

    public void createMessageForm() {
        try {
            MamManager mamManager = MamManager.getInstanceFor(connect);
            //EntityBareJid jid = JidCreate.entityBareFrom("evgeny.polivanov@msg.sharix-app.org");
            MamManager.MamQueryArgs mamQueryArgs = MamManager.MamQueryArgs.builder()
                    .limitResultsToJid(jid)
                    .setResultPageSizeTo(10)
                    .queryLastPage()
                    .build();
            MamManager.MamQuery mamQuery = mamManager.queryArchive(mamQueryArgs);
            List<Message> messages = mamQuery.getMessages();
            for(Message message : messages){
                String from = message.getFrom().asBareJid().toString().trim();
                if (from.equals(jid.asEntityBareJidString().trim())){
                    addMessage(message.getFrom().asBareJid().toString(), message.getBody(), true);
                }
                else {
                    addMessage(message.getFrom().asBareJid().toString(), message.getBody(), false);
                }

            }
            runMessageListener();

        }catch (Exception ex){

        }

    }

    public void runMessageListener(){
        chatManager.addIncomingListener(new IncomingChatMessageListener() {
            @Override
            public void newIncomingMessage(EntityBareJid from, Message message, Chat chat) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        addMessage(from.asBareJid().toString(), message.getBody(), true);
                        System.out.println("New message from:" + from + ": " + message.toXML().toString());
                    }
                });

            }
        });
    }
    public void addMessage(String from, String body, boolean red){
        TextView fromName = new TextView(this);
        TextView bodyView = new TextView(this);
        LinearLayout linearLayout = new LinearLayout(this);

        fromName.setText(from + ": ");
        fromName.setTextSize(16);
        fromName.setTypeface(null, Typeface.BOLD);
        if (red){
            fromName.setTextColor(Color.parseColor("#a51b0b"));
        } else {
            fromName.setTextColor(Color.parseColor("#006000"));
        }


        bodyView.setText(body);
        bodyView.setTextSize(14);
        bodyView.setTextColor(Color.parseColor("#000000"));

        linearLayout.addView(fromName);
        linearLayout.addView(bodyView);

        chatLayout.addView(linearLayout);
    }
}